package cn.ibizlab.eam.core.eam_core.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.eam_core.domain.EMPlan;
import cn.ibizlab.eam.core.eam_core.filter.EMPlanSearchContext;
import cn.ibizlab.eam.core.eam_core.service.IEMPlanService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.eam_core.mapper.EMPlanMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[计划] 服务对象接口实现
 */
@Slf4j
@Service("EMPlanServiceImpl")
public class EMPlanServiceImpl extends ServiceImpl<EMPlanMapper, EMPlan> implements IEMPlanService {

    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMPlanCDTService emplancdtService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMPlanDetailService emplandetailService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMACClassService emacclassService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMEquipService emequipService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMObjectService emobjectService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMPlanTemplService emplantemplService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMServiceService emserviceService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFDeptService pfdeptService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFEmpService pfempService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFTeamService pfteamService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EMPlan et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEmplanid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EMPlan> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EMPlan et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("emplanid",et.getEmplanid())))
            return false;
        CachedBeanCopier.copy(get(et.getEmplanid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EMPlan> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public EMPlan get(String key) {
        EMPlan et = getById(key);
        if(et==null){
            et=new EMPlan();
            et.setEmplanid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    public EMPlan getDraft(EMPlan et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(EMPlan et) {
        return (!ObjectUtils.isEmpty(et.getEmplanid()))&&(!Objects.isNull(this.getById(et.getEmplanid())));
    }

    @Override
    @Transactional
    public boolean save(EMPlan et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EMPlan et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EMPlan> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EMPlan> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EMPlan> selectByAcclassid(String emacclassid) {
        return baseMapper.selectByAcclassid(emacclassid);
    }

    @Override
    public void removeByAcclassid(String emacclassid) {
        this.remove(new QueryWrapper<EMPlan>().eq("acclassid",emacclassid));
    }

	@Override
    public List<EMPlan> selectByEquipid(String emequipid) {
        return baseMapper.selectByEquipid(emequipid);
    }

    @Override
    public void removeByEquipid(String emequipid) {
        this.remove(new QueryWrapper<EMPlan>().eq("equipid",emequipid));
    }

	@Override
    public List<EMPlan> selectByDpid(String emobjectid) {
        return baseMapper.selectByDpid(emobjectid);
    }

    @Override
    public void removeByDpid(String emobjectid) {
        this.remove(new QueryWrapper<EMPlan>().eq("dpid",emobjectid));
    }

	@Override
    public List<EMPlan> selectByObjid(String emobjectid) {
        return baseMapper.selectByObjid(emobjectid);
    }

    @Override
    public void removeByObjid(String emobjectid) {
        this.remove(new QueryWrapper<EMPlan>().eq("objid",emobjectid));
    }

	@Override
    public List<EMPlan> selectByPlantemplid(String emplantemplid) {
        return baseMapper.selectByPlantemplid(emplantemplid);
    }

    @Override
    public void removeByPlantemplid(String emplantemplid) {
        this.remove(new QueryWrapper<EMPlan>().eq("plantemplid",emplantemplid));
    }

	@Override
    public List<EMPlan> selectByRserviceid(String emserviceid) {
        return baseMapper.selectByRserviceid(emserviceid);
    }

    @Override
    public void removeByRserviceid(String emserviceid) {
        this.remove(new QueryWrapper<EMPlan>().eq("rserviceid",emserviceid));
    }

	@Override
    public List<EMPlan> selectByRdeptid(String pfdeptid) {
        return baseMapper.selectByRdeptid(pfdeptid);
    }

    @Override
    public void removeByRdeptid(String pfdeptid) {
        this.remove(new QueryWrapper<EMPlan>().eq("rdeptid",pfdeptid));
    }

	@Override
    public List<EMPlan> selectByMpersonid(String pfempid) {
        return baseMapper.selectByMpersonid(pfempid);
    }

    @Override
    public void removeByMpersonid(String pfempid) {
        this.remove(new QueryWrapper<EMPlan>().eq("mpersonid",pfempid));
    }

	@Override
    public List<EMPlan> selectByRecvpersonid(String pfempid) {
        return baseMapper.selectByRecvpersonid(pfempid);
    }

    @Override
    public void removeByRecvpersonid(String pfempid) {
        this.remove(new QueryWrapper<EMPlan>().eq("recvpersonid",pfempid));
    }

	@Override
    public List<EMPlan> selectByRempid(String pfempid) {
        return baseMapper.selectByRempid(pfempid);
    }

    @Override
    public void removeByRempid(String pfempid) {
        this.remove(new QueryWrapper<EMPlan>().eq("rempid",pfempid));
    }

	@Override
    public List<EMPlan> selectByRteamid(String pfteamid) {
        return baseMapper.selectByRteamid(pfteamid);
    }

    @Override
    public void removeByRteamid(String pfteamid) {
        this.remove(new QueryWrapper<EMPlan>().eq("rteamid",pfteamid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EMPlan> searchDefault(EMPlanSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMPlan> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMPlan>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EMPlan et){
        //实体关系[DER1N_EMPLAN_EMACCLASS_ACCLASSID]
        if(!ObjectUtils.isEmpty(et.getAcclassid())){
            cn.ibizlab.eam.core.eam_core.domain.EMACClass acclass=et.getAcclass();
            if(ObjectUtils.isEmpty(acclass)){
                cn.ibizlab.eam.core.eam_core.domain.EMACClass majorEntity=emacclassService.get(et.getAcclassid());
                et.setAcclass(majorEntity);
                acclass=majorEntity;
            }
            et.setAcclassname(acclass.getEmacclassname());
        }
        //实体关系[DER1N_EMPLAN_EMEQUIP_EQUIPID]
        if(!ObjectUtils.isEmpty(et.getEquipid())){
            cn.ibizlab.eam.core.eam_core.domain.EMEquip equip=et.getEquip();
            if(ObjectUtils.isEmpty(equip)){
                cn.ibizlab.eam.core.eam_core.domain.EMEquip majorEntity=emequipService.get(et.getEquipid());
                et.setEquip(majorEntity);
                equip=majorEntity;
            }
            et.setEquipname(equip.getEmequipname());
        }
        //实体关系[DER1N_EMPLAN_EMOBJECT_DPID]
        if(!ObjectUtils.isEmpty(et.getDpid())){
            cn.ibizlab.eam.core.eam_core.domain.EMObject dp=et.getDp();
            if(ObjectUtils.isEmpty(dp)){
                cn.ibizlab.eam.core.eam_core.domain.EMObject majorEntity=emobjectService.get(et.getDpid());
                et.setDp(majorEntity);
                dp=majorEntity;
            }
            et.setDpname(dp.getEmobjectname());
            et.setDptype(dp.getEmobjecttype());
        }
        //实体关系[DER1N_EMPLAN_EMOBJECT_OBJID]
        if(!ObjectUtils.isEmpty(et.getObjid())){
            cn.ibizlab.eam.core.eam_core.domain.EMObject obj=et.getObj();
            if(ObjectUtils.isEmpty(obj)){
                cn.ibizlab.eam.core.eam_core.domain.EMObject majorEntity=emobjectService.get(et.getObjid());
                et.setObj(majorEntity);
                obj=majorEntity;
            }
            et.setObjname(obj.getEmobjectname());
        }
        //实体关系[DER1N_EMPLAN_EMPLANTEMPL_PLANTEMPLID]
        if(!ObjectUtils.isEmpty(et.getPlantemplid())){
            cn.ibizlab.eam.core.eam_core.domain.EMPlanTempl plantempl=et.getPlantempl();
            if(ObjectUtils.isEmpty(plantempl)){
                cn.ibizlab.eam.core.eam_core.domain.EMPlanTempl majorEntity=emplantemplService.get(et.getPlantemplid());
                et.setPlantempl(majorEntity);
                plantempl=majorEntity;
            }
            et.setPlantemplname(plantempl.getEmplantemplname());
        }
        //实体关系[DER1N_EMPLAN_EMSERVICE_RSERVICEID]
        if(!ObjectUtils.isEmpty(et.getRserviceid())){
            cn.ibizlab.eam.core.eam_core.domain.EMService rservice=et.getRservice();
            if(ObjectUtils.isEmpty(rservice)){
                cn.ibizlab.eam.core.eam_core.domain.EMService majorEntity=emserviceService.get(et.getRserviceid());
                et.setRservice(majorEntity);
                rservice=majorEntity;
            }
            et.setRservicename(rservice.getEmservicename());
        }
        //实体关系[DER1N_EMPLAN_PFDEPT_RDEPTID]
        if(!ObjectUtils.isEmpty(et.getRdeptid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFDept rdept=et.getRdept();
            if(ObjectUtils.isEmpty(rdept)){
                cn.ibizlab.eam.core.eam_pf.domain.PFDept majorEntity=pfdeptService.get(et.getRdeptid());
                et.setRdept(majorEntity);
                rdept=majorEntity;
            }
            et.setRdeptname(rdept.getPfdeptname());
        }
        //实体关系[DER1N_EMPLAN_PFEMP_MPERSONID]
        if(!ObjectUtils.isEmpty(et.getMpersonid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEmp mperson=et.getMperson();
            if(ObjectUtils.isEmpty(mperson)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEmp majorEntity=pfempService.get(et.getMpersonid());
                et.setMperson(majorEntity);
                mperson=majorEntity;
            }
            et.setMpersonname(mperson.getPfempname());
        }
        //实体关系[DER1N_EMPLAN_PFEMP_RECVPERSONID]
        if(!ObjectUtils.isEmpty(et.getRecvpersonid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEmp recvperson=et.getRecvperson();
            if(ObjectUtils.isEmpty(recvperson)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEmp majorEntity=pfempService.get(et.getRecvpersonid());
                et.setRecvperson(majorEntity);
                recvperson=majorEntity;
            }
            et.setRecvpersonname(recvperson.getPfempname());
        }
        //实体关系[DER1N_EMPLAN_PFEMP_REMPID]
        if(!ObjectUtils.isEmpty(et.getRempid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEmp remp=et.getRemp();
            if(ObjectUtils.isEmpty(remp)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEmp majorEntity=pfempService.get(et.getRempid());
                et.setRemp(majorEntity);
                remp=majorEntity;
            }
            et.setRempname(remp.getPfempname());
        }
        //实体关系[DER1N_EMPLAN_PFTEAM_RTEAMID]
        if(!ObjectUtils.isEmpty(et.getRteamid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFTeam rteam=et.getRteam();
            if(ObjectUtils.isEmpty(rteam)){
                cn.ibizlab.eam.core.eam_pf.domain.PFTeam majorEntity=pfteamService.get(et.getRteamid());
                et.setRteam(majorEntity);
                rteam=majorEntity;
            }
            et.setRteamname(rteam.getPfteamname());
        }
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EMPlan> getEmplanByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EMPlan> getEmplanByEntities(List<EMPlan> entities) {
        List ids =new ArrayList();
        for(EMPlan entity : entities){
            Serializable id=entity.getEmplanid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}


