package cn.ibizlab.eam.core.eam_core.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.eam_core.domain.EMPlanTDetail;
import cn.ibizlab.eam.core.eam_core.filter.EMPlanTDetailSearchContext;
import cn.ibizlab.eam.core.eam_core.service.IEMPlanTDetailService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.eam_core.mapper.EMPlanTDetailMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[计划模板步骤] 服务对象接口实现
 */
@Slf4j
@Service("EMPlanTDetailServiceImpl")
public class EMPlanTDetailServiceImpl extends ServiceImpl<EMPlanTDetailMapper, EMPlanTDetail> implements IEMPlanTDetailService {

    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_core.service.IEMPlanTemplService emplantemplService;
    @Autowired
    @Lazy
    private cn.ibizlab.eam.core.eam_pf.service.IPFEmpService pfempService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean update(EMPlanTDetail et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("emplantdetailid",et.getEmplantdetailid())))
            return false;
        CachedBeanCopier.copy(get(et.getEmplantdetailid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EMPlanTDetail> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    public EMPlanTDetail getDraft(EMPlanTDetail et) {
        fillParentData(et);
        return et;
    }

    @Override
    @Transactional
    public boolean create(EMPlanTDetail et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEmplantdetailid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EMPlanTDetail> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean save(EMPlanTDetail et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(EMPlanTDetail et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EMPlanTDetail> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EMPlanTDetail> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public EMPlanTDetail get(String key) {
        EMPlanTDetail et = getById(key);
        if(et==null){
            et=new EMPlanTDetail();
            et.setEmplantdetailid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    public boolean checkKey(EMPlanTDetail et) {
        return (!ObjectUtils.isEmpty(et.getEmplantdetailid()))&&(!Objects.isNull(this.getById(et.getEmplantdetailid())));
    }


	@Override
    public List<EMPlanTDetail> selectByPlantemplid(String emplantemplid) {
        return baseMapper.selectByPlantemplid(emplantemplid);
    }

    @Override
    public void removeByPlantemplid(String emplantemplid) {
        this.remove(new QueryWrapper<EMPlanTDetail>().eq("plantemplid",emplantemplid));
    }

	@Override
    public List<EMPlanTDetail> selectByRecvpersonid(String pfempid) {
        return baseMapper.selectByRecvpersonid(pfempid);
    }

    @Override
    public void removeByRecvpersonid(String pfempid) {
        this.remove(new QueryWrapper<EMPlanTDetail>().eq("recvpersonid",pfempid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EMPlanTDetail> searchDefault(EMPlanTDetailSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EMPlanTDetail> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EMPlanTDetail>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EMPlanTDetail et){
        //实体关系[DER1N_EMPLANTDETAIL_EMPLANTEMPL_PLANTEMPLID]
        if(!ObjectUtils.isEmpty(et.getPlantemplid())){
            cn.ibizlab.eam.core.eam_core.domain.EMPlanTempl plantempl=et.getPlantempl();
            if(ObjectUtils.isEmpty(plantempl)){
                cn.ibizlab.eam.core.eam_core.domain.EMPlanTempl majorEntity=emplantemplService.get(et.getPlantemplid());
                et.setPlantempl(majorEntity);
                plantempl=majorEntity;
            }
            et.setPlantemplname(plantempl.getEmplantemplname());
        }
        //实体关系[DER1N_EMPLANTDETAIL_PFEMP_RECVPERSONID]
        if(!ObjectUtils.isEmpty(et.getRecvpersonid())){
            cn.ibizlab.eam.core.eam_pf.domain.PFEmp recvperson=et.getRecvperson();
            if(ObjectUtils.isEmpty(recvperson)){
                cn.ibizlab.eam.core.eam_pf.domain.PFEmp majorEntity=pfempService.get(et.getRecvpersonid());
                et.setRecvperson(majorEntity);
                recvperson=majorEntity;
            }
            et.setRecvpersonname(recvperson.getPfempname());
        }
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EMPlanTDetail> getEmplantdetailByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EMPlanTDetail> getEmplantdetailByEntities(List<EMPlanTDetail> entities) {
        List ids =new ArrayList();
        for(EMPlanTDetail entity : entities){
            Serializable id=entity.getEmplantdetailid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}


