package cn.ibizlab.eam.core.r7rt_dyna.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.eam.core.r7rt_dyna.domain.BXD;
import cn.ibizlab.eam.core.r7rt_dyna.filter.BXDSearchContext;
import cn.ibizlab.eam.core.r7rt_dyna.service.IBXDService;

import cn.ibizlab.eam.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.eam.core.r7rt_dyna.mapper.BXDMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[报销单] 服务对象接口实现
 */
@Slf4j
@Service("BXDServiceImpl")
public class BXDServiceImpl extends ServiceImpl<BXDMapper, BXD> implements IBXDService {


    private int batchSize = 500;

    @Override
    @Transactional
    public BXD get(String key) {
        BXD et = getById(key);
        if(et==null){
            et=new BXD();
            et.setBxdid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean update(BXD et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("bxdid",et.getBxdid())))
            return false;
        CachedBeanCopier.copy(get(et.getBxdid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<BXD> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean create(BXD et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getBxdid()),et);
        return true;
    }

    @Override
    public void createBatch(List<BXD> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean save(BXD et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(BXD et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<BXD> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<BXD> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    public boolean checkKey(BXD et) {
        return (!ObjectUtils.isEmpty(et.getBxdid()))&&(!Objects.isNull(this.getById(et.getBxdid())));
    }

    @Override
    public BXD getDraft(BXD et) {
        return et;
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<BXD> searchDefault(BXDSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<BXD> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<BXD>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<BXD> getBxdByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<BXD> getBxdByEntities(List<BXD> entities) {
        List ids =new ArrayList();
        for(BXD entity : entities){
            Serializable id=entity.getBxdid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}


